#!/bin/bash

INFO_MESSAGE=0
SEND_EMAIL=0
DISK_LOW_SIZE='2.0G'


 ## mnt_* Info:
 ## name             - mount point name
 ## path             - path to folder with mount point
 ## regex            - regular expression for the find utility
 ## limit_delete_low - lower limit of deleted files if disk size is less than $DISK_LOW_SIZE
 ## limit_count_low  - lower limit count of files before deletion if disk size is less than $DISK_LOW_SIZE
 ## limit_delete_low - high limit of deleted files if disk size is more than $DISK_LOW_SIZE
 ## limit_count_low  - high limit count of files before deletion if disk size is more than $DISK_LOW_SIZE

declare -A mnt_01=(
    ['name']="Backups"
    ['path']="/mnt/backups"
    ['regex']=".*\.\(tar.gz\)$"
    ['limit_delete_low']="5"
    ['limit_count_low']="15"
    ['limit_delete_hi']="10"
    ['limit_count_hi']="30"
  )
declare -A mnt_02=(
    ['name']="Faxes"
    ['path']="/mnt/faxes"
    ['regex']=".*\.\(tiff\|pdf\)$"
    ['limit_delete_low']="60"
    ['limit_count_low']="120"
    ['limit_delete_hi']="300"
    ['limit_count_hi']="600"
  )
declare -A mnt_03=(
    ['name']="Call recording"
    ['path']="/dev/shm/var/spool/callweaver/monitor"
    ['regex']=".*\.\(mp3\|wav\|alaw\)$"
    ['limit_delete_low']="80"
    ['limit_count_low']="120"
    ['limit_delete_hi']="400"
    ['limit_count_hi']="600"
  )
declare -A mnt_04=(
    ['name']="Voicemail"
    ['path']="/dev/shm/var/spool/callweaver/voicemail"
    ['regex']=".*\.\(mp3\|wav\|alaw\)$"
    ['limit_delete_low']="80"
    ['limit_count_low']="120"
    ['limit_delete_hi']="400"
    ['limit_count_hi']="600"
  )
declare -A mnt_05=(
    ['name']="CDR CSV"
    ['path']="/mnt/cdrcsv"
  )
declare -A mnt_06=(
    ['name']="CDR SQLite"
    ['path']="/mnt/cdr"
  )
declare -A mnt_07=(
    ['name']="CTI"
    ['path']="/mnt/cticonnect"
  )

declare -a CLEANUP_MOUNTS=(mnt_01 mnt_02 mnt_03 mnt_04 mnt_05 mnt_06 mnt_07)


__LOG() {

  local name="${0##*/}"
  local message="${2}"
  local xargs="${3}"

  case "${1}" in
    warn) severity='warning' ;;
    err) severity='err' ;;
    *) severity='info' ;;
  esac

  if [ "${message}" ]; then
    echo ${xargs} "${message}" | tee >(systemd-cat -t "${name}" -p "${severity}")
  else
    systemd-cat -t "${name}" -p "${severity}"
  fi

}


__SEND_MAIL(){

  local mail_flag mail_config mail_warn_msg mail_from mail_admin pbx_name mailbody mail_message mail_wait_time

  mail_flag='/tmp/cleanup_msg_lock' # path to lock file
  mail_wait_time='10800'            # waiting (seconds) before try next send email

  if [ -f "${mail_flag}" ]; then
    [ "$(($(date +%s) - $(stat -c %Y "${mail_flag}")))" -ge "${mail_wait_time}" ] && >${mail_flag} || return 0
  else
    >${mail_flag}
  fi

    ## include script for ldap
  if [ -s '/usr/sbin/ldapcmd' ]; then
    source /usr/sbin/ldapcmd
  else
    __LOG "err" "ERROR($FUNCNAME): /usr/sbin/ldapcmd was not found." && return 1
  fi

  mail_message="${1}"
  mail_warn_msg="${2:-'10%'}"
  mail_config='/etc/companies.d/0/msmtprc'
  mail_from="$(awk '/from/{print $2}' $mail_config)"
  mail_admin="$(__LDAP_GET "(&(objectClass=wPerson)(cn=admin)(userKind=admin)) mail||mail" "dc=wildix" | awk '{print $2}')"
  pbx_name="$(cat /etc/hostname)"

    # check mail password and admin mail
  [ -z "${mail_admin}" ] || [ -z "$(awk '/password/{print $2}' ${mail_config})" ] && \
    __LOG "err" "ERROR($FUNCNAME): not set password or admin email" && return 1

    # check from
  [ -z "${mail_from}" ] && mail_from="no-reply@wildix.com"

    # send email
  mailbody="The following points have disk space less than ${mail_warn_msg}:\n\n${mail_message}\nPlease clear the space for the correct operation of the system."

  /usr/sbin/sendemail.php \
    --type="default" \
    --body="$(echo -e ${mailbody})" \
    --subject="Free disk space problem on ${pbx_name}.wildixin.com" \
    --to=${mail_admin}

  [ "${?}" -eq "0" ] && __LOG "info" "Notification has been sent to ${mail_admin}." || \
    __LOG "err" "ERROR($FUNCNAME): Notification hasn't been delivered due to an error to ${mail_admin}."

}


__GET_LIMITS(){

  local mnt_size mailing_procent cleaning_procent max_size mailing_size cleaning_size arr

  declare -n arr="${1}"
  mnt_size="${2}"
  mailing_procent="${3:-10}"
  cleaning_procent="${4:-5}"
  max_size="$(numfmt --from=iec --to-unit=1024 ${5:-15G})"

    ## check local variables for empty values
  [ -z "${!arr}" ] || [ -z "${mnt_size}" ] && \
    __LOG "err" "ERROR($FUNCNAME): returned data is empty." && return 1

    ## if size of disk more then ${max_size}
  if [ "${mnt_size}" -gt "${max_size}" ]; then
    mailing_size="$(numfmt --from=iec --to-unit=1024 1.2G)"
    cleaning_size="$(numfmt --from=iec --to-unit=1024 1G)"
    OVERRIDE_WARN_MSG="$(numfmt --to=iec --from-unit=1024 ${mailing_size})"
    ## if size of disk less then ${max_size}
  else
    mailing_size="$(awk "BEGIN {printf \"%.0f\n\", (${mailing_procent}/100)*${mnt_size}}")"
    cleaning_size="$(awk "BEGIN {printf \"%.0f\n\", (${cleaning_procent}/100)*${mnt_size}}")"
  fi

    ## overrides cleaning limits
    if [ "${OVERRIDE_SIZE}" ]; then

      if [ -f '/usr/sbin/wildix_shell_tools' ]; then
        . /usr/sbin/wildix_shell_tools
      else
        __LOG "err" "ERROR($FUNCNAME): can't include the wildix_shell_tools" && return 1
      fi

      [ "$(is_cloud)" -eq 1 ] && cleaning_size="$(numfmt --from=iec --to-unit=1024 ${OVERRIDE_SIZE})" || \
        __LOG "warn" "WARNING($FUNCNAME): this PBX is not in the cloud you can't override a size of cleanup."
    fi

  arr['mailing_size']="${mailing_size}"
  arr['cleaning_size']="${cleaning_size}"

}


__GET_DISK_INFO(){

  local arr mnt_data

  declare -n arr="${1}"
  mnt_data="$(timeout 5 df -Pa ${2} 2>/dev/null | sed '1d')"

    ## check local variables for empty values
  [ -z "${!arr}" ] || [ -z "${mnt_data}" ] && \
    __LOG "err" "ERROR($FUNCNAME): a disk util returns empty data - (${mnt_point})." && return 1

  arr['size']="$(awk '{print $2}' <<< ${mnt_data})"
  arr['used']="$(awk '{print $3}' <<< ${mnt_data})"
  arr['available']="$(awk '{print $4}' <<< ${mnt_data})"
  arr['used%']="$(awk '{print $5}' <<< ${mnt_data})"

}


__CLEANUP(){

  local mnt disk_info mnt_get_files limit_delete limit_count limits deleted_files

    ## get arguments
  declare -n mnt="${1}"
  [ "${#mnt[@]}" -eq 0 ] && \
  __LOG "err" "ERROR($FUNCNAME): mnt information is empty - (${mnt['path']})." && return 1

    ## check if we have a regular expression to cleanup
  if [ "${mnt['regex']}" ]; then

      ## get disk parameters
    declare -A disk_info && __GET_DISK_INFO disk_info "${mnt['path']}"
    [ "${#disk_info[@]}" -eq 0 ] && \
      __LOG "err" "ERROR($FUNCNAME): mount point information is empty - (${mnt['path']})." && return 1

      ## gets limits from mnt size
    declare -A limits && __GET_LIMITS limits "${disk_info['size']}"
    [ "${#limits[@]}" -eq 0 ] && \
      __LOG "err" "ERROR($FUNCNAME): disk limits is empty - (${mnt['path']})." && return 1

      ## check local variables for empty values
    [ -z "${mnt['limit_delete_low']}" ] || [ -z "${mnt['limit_count_low']}" ] || [ -z "${mnt['limit_delete_hi']}" ] || [ -z "${mnt['limit_count_hi']}" ] && \
      __LOG "err" "ERROR($FUNCNAME): returned data is empty - (${mnt['path']})." && return 1

      ## selects limits (compares size with $DISK_LOW_SIZE)
    if [ "${disk_info['size']}" -le "$(numfmt --from=iec --to-unit=1024 ${DISK_LOW_SIZE})" ]; then
      limit_delete="${mnt['limit_delete_low']}"
      limit_count="${mnt['limit_count_low']}"
    else
      limit_delete="${mnt['limit_delete_hi']}"
      limit_count="${mnt['limit_count_hi']}"
    fi

      ## send email
    if [ "${disk_info['available']}" -le "${limits['mailing_size']}" ] && [ "${disk_info['available']}" -gt "${limits['cleaning_size']}" ]; then
      if [ -z "${EMAIL_LOCK}" ]; then
        mnt_status="email" && SEND_EMAIL=1
      else
        mnt_status="email_(skip)" && SEND_EMAIL=0
      fi

      ## perform cleanup
    elif [ "${disk_info['available']}" -le "${limits['cleaning_size']}" ]; then
      declare -a mnt_get_files=(
          $(timeout 10 find ${mnt['path']} -type f -regex ${mnt['regex']} -printf "%T@ %p\n" | sort -n | awk '{print $2}')
        )

        ## if there are enough files, delete
      if [ "${#mnt_get_files[@]}" -ge "${limit_count}" ]; then

        xargs rm -f  {} <<< ${mnt_get_files[@]:0:${limit_delete}}
          ## clean up empty folders
        timeout 10 find ${mnt['path']} -type d -empty -not -path '*/uploads*' -not -path '*/invoices*' -not -path '*/lost+found*' -not -path '*/traces*' -delete

        __LOG "info" "In the directory (${mnt['path']}) have been removed: (${mnt_get_files[*]:0:${limit_delete}})" '-e'
        mnt_status="cleaned_(${limit_delete})"
          ## lock send email
        EMAIL_LOCK=1

      else
        mnt_status="skiped_(${#mnt_get_files[@]}<${limit_count})"
      fi
    else
      mnt_status="ok"
    fi

      ## status message
    STATUS_MESSAGE+="${mnt['path']} ${disk_info['size']} ${disk_info['available']} ${limits['mailing_size']} ${limits['cleaning_size']} ${mnt_status}\n"
      ## generate email message for notifications
    MAIL_MESSAGE+="${mnt['name']} - ${disk_info['used%']}\n"

  fi

}

if [ "$(pidof -x cleaning_space.sh | wc -w)" -gt 3 ]; then
  __LOG "err" "SPACE CLEANING: Another process already running. Wait..."
  exit 0
fi

case "${1}" in
  info) INFO_MESSAGE=1 ;;
  *)
    if [[ "${1}" =~ ^[0-9|.]{1,5}[a-z|A-Z]{1}$ ]]; then
      OVERRIDE_SIZE="$(tr 'a-z' 'A-Z' <<< ${1})"
    fi
  ;;
esac

for key in ${!CLEANUP_MOUNTS[@]}; do
  __CLEANUP "${CLEANUP_MOUNTS[${key}]}"
done

  ## display information
[ "${INFO_MESSAGE}" -eq 1 ] && \
  echo -e "path: size: available: mail-trigger: cleanup-trigger: status:\n${STATUS_MESSAGE}" |\
  numfmt --to=iec --from-unit=1024 --header --field=2-5  | column -t

  ## sent email to admin if ${SEND_EMAIL} is 1
[ "${SEND_EMAIL}" -eq 1 ] && [ "${EMAIL_LOCK:-0}" -ne 1 ] && __SEND_MAIL "${MAIL_MESSAGE}" "${OVERRIDE_WARN_MSG}"
